<?php
namespace Teknisa\SUP\Modules\Cotacao\Modules\PlanilhaCotacao\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class Sup07000LogDeCotacaoController extends \Zeedhi\Framework\Controller\Report
{
    protected $entityManager;
    protected $qr2Strategy;
    protected $environment;
    protected $birtStrategy;
    protected $newCode;
    protected $connection;
    
    public function __construct($reportService, $entityManager, $qr2Strategy, $environment, $birtStrategy, $newCode, $connection)
    {
        $this->entityManager       = $entityManager;
        $this->qr2Strategy         = $qr2Strategy;
        $this->reportService       = $reportService;
        $this->environment         = $environment;
        $this->birtStrategy        = $birtStrategy;
        $this->newCode             = $newCode;
        $this->connection          = $connection;
    }
        
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $row['NRORG'] = $this->environment->getNrOrg();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->stNomeRel = $this->getReportName($row);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        if($row['OPERADOR'] != 'T'){
                 $nrlistaForn= $this->criarListaValores($row['OPERADOR']);
                 $row['OPERADOR'] = "_NRLISTA=".$nrlistaForn;
        }
        
        $this->reportMapping[$this->stNomeRel] =
            array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                "OPERADOR"      => "L_CDOPERADOR",
                "P_DATAINIC"    => "P_DATAINIC",
                "P_DATAFINA"    => "P_DATAFINA",
                "DTINI"         => "P_DTI",
                "DTFIM"         => "P_DTF",
                "NR_COTACAO"    => "P_NRCOLEPRLOG",
                "NRORG"         => "P_NRORG"
                
            ),
        );
    }
    
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

     protected function criarListaValores($lista){
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_',$lista);
        try{
            foreach($valores as $key){
                $this->insertLista($nrlista, $key);
            }
            return $nrlista;
        } catch(\Exception $e){
            throw $e;
        }
     }
}
