<?php

namespace Teknisa\SUP\Modules\Cotacao\Modules\PlanilhaCotacao\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO;
use Teknisa\SUP\Util\SupUtilities;

class Sup10700PlanilhaDePrecoObjetivoService {
    protected $entityManager;
    protected $environment;
    protected $connection;
    
    public function __construct($entityManager, $environment, $connection, SupUtilities $supUtilities) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->connection    = $connection;
        $this->supUtilities  = $supUtilities;
    }
    
    /********************
     *      SELECTS     *
     * ******************/
     
    public function getFilial($params) 
    {
		$param = array(
		    'P_CDFILIAL'  => $params['P_CDFILIAL'],
		    'P_TCDFILIAL' => $params['P_TCDFILIAL'],
		    'P_NRORG'     => $this->environment->getNrorg()
		);
		
        $query = "SELECT CDFILIAL, CDREGIAO
                    FROM FILIAL
                   WHERE (INSTR(:P_CDFILIAL, CDFILIAL) > 0 OR (:P_TCDFILIAL = 'T'))
                     AND CDREGIAO IS NOT NULL
                    AND NRORG = :P_NRORG
                   ORDER BY CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function strPadRigthProd($value){
	    return str_pad($value, 20, 'Z', STR_PAD_RIGHT);
	}
    
    public function getProdSolicita($params) 
    {
		$param = array(
		    'P_CDFILIAL'      => $params['P_CDFILIAL'],
		    'P_TCDFILIAL'     => $params['P_TCDFILIAL'],
		    'P_TODOSPRODUTOS' => $params['P_TODOSPRODUTOS'],
		    'P_DTINIVALOBJET' => $params['P_DTINIVALOBJET'],
		    'P_DTFINVALOBJET' => $params['P_DTFINVALOBJET'],
		    'P_PRODINI'       => $params['P_PRODINI'],
		    'P_PRODFIN'       => $params['P_PRODFIN'],
		    'P_NRORG'         => $this->environment->getNrorg()
		);

        $query = "SELECT DISTINCT I.CDFILSOLI CDFILIAL, P.CDPRODESTO CDPRODUTO, F.CDREGIAO
                    FROM ITEMSOLI I, PRODUTO P, FILIAL F
                   WHERE (INSTR(:P_CDFILIAL, I.CDFILSOLI) >0 OR (:P_TCDFILIAL = 'T'))
                     AND (I.DTPROGENTR BETWEEN :P_DTINIVALOBJET AND :P_DTFINVALOBJET)
                     AND ((P.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN) OR (:P_TODOSPRODUTOS = 'T'))
                     AND I.CDPRODUTO = P.CDPRODUTO
                     AND I.CDFILSOLI = F.CDFILIAL
                     AND NVL(I.IDSOLIAPROV, 'N') = 'S'
                     AND I.NRORG = :P_NRORG
                ORDER BY P.CDPRODESTO";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            return $std->fetch();
        } catch (Exception $e){
            throw $e;
        }
    }

    public function getUltPrecoObjetivo($params) 
    {
		$param = array(
		    'P_CDFILIAL'  => $params['P_CDFILIAL'],
		    'P_CDREGIAO'  => $params['P_CDREGIAO'],
		    'P_CDPRODUTO' => $params['P_CDPRODUTO'],
		    'P_NRORG'     => $this->environment->getNrorg()
		);
		
        $query = "SELECT MIN(VRPRECOBJETIVO) VRPRECOBJETIVO
                    FROM ITEMPRECOJETIVO
                   WHERE CDFILIAL  = :P_CDFILIAL
                     AND CDREGIAO  = :P_CDREGIAO
                     AND CDPRODUTO = :P_CDPRODUTO
                     AND NRORG     = :P_NRORG
                     AND DTFINVALITMOBJ = (SELECT MAX(DTFINVALITMOBJ)
                                             FROM ITEMPRECOJETIVO I
                                            WHERE CDFILIAL  = :P_CDFILIAL
                                              AND CDREGIAO  = :P_CDREGIAO
                                              AND CDPRODUTO = :P_CDPRODUTO )";
                        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            return $std->fetch();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getProduto($params) 
    {
		$param = array(
		    'P_PRODINI'       => $params['P_PRODINI'],
		    'P_PRODFIN'       => $params['P_PRODFIN'] = str_pad($params['P_PRODFIN'], 20, 'Z', STR_PAD_RIGHT),
		    'P_TODOSPRODUTOS' => $params['P_PRODINI'] ? null: $params['P_TODOSPRODUTOS'],
		    'P_NRORG'         => $this->environment->getNrorg()
		);
		
        $query = "SELECT CDPRODESTO CDPRODUTO
                    FROM PRODUTO
                   WHERE ((CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN) OR (:P_TODOSPRODUTOS = 'T'))
                     AND (CDPRODUTO = CDPRODESTO)
                     AND (NRORG     = :P_NRORG)
                   ORDER BY CDPRODUTO";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            return $std->fetchAll();
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function getVdPlanPrecObj($params) 
    {
		$param = array(
		    'P_CDFILIAL'       => $params['P_CDFILIAL'],
		    'P_NRPRECOBJET'    => $params['P_NRPRECOBJET'],
		    'P_DTINIVALITMOBJ' => $params['P_DTINIVALOBJET'],
		    'P_DTFINVALITMOBJ' => $params['P_DTFINVALOBJET'],
		    'P_NRORG'          => $this->environment->getNrorg()
		);
        $query = "SELECT COUNT(CDFILIAL) QUANTPRECOBJ
                    FROM PRECOBJETIVO
                   WHERE CDFILIAL      = :P_CDFILIAL
                     AND NRPRECOBJET   = :P_NRPRECOBJET
                     AND DTINIVALOBJET = :P_DTINIVALITMOBJ
                     AND DTFINVALOBJET = :P_DTFINVALITMOBJ
                     AND NRORG         = :P_NRORG
                     AND EXISTS (SELECT 1
                                   FROM ITEMPRECOJETIVO
                                  WHERE CDFILIAL       = :P_CDFILIAL
                                    AND NRPRECOBJET    = :P_NRPRECOBJET
                                    AND DTINIVALITMOBJ = :P_DTINIVALITMOBJ
                                    AND DTFINVALITMOBJ = :P_DTFINVALITMOBJ)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            return $std->fetch();
        } catch (Exception $e){
            throw $e;
        }
    }
    

    /********************
     *      INSERTS     *
     * ******************/
     
    public function insertPlanObj($params){    
        try {
            $query = "INSERT INTO PRECOBJETIVO (CDFILIAL, NRPRECOBJET, DTPRECOBJET, DTINIVALOBJET, DTFINVALOBJET, NRORG) VALUES (:CDFILIAL, :NRPRECOBJET, SYSDATE, :DTINIVALOBJET, :DTFINVALOBJET, :P_NRORG)";
           
            $param = array (
    		    'CDFILIAL'      => $params['P_CDFILIAL'],
    		    'NRPRECOBJET'   => $params['P_NRPRECOBJET'],
    		    'DTINIVALOBJET' => $params['P_DTINIVALOBJET'],
    		    'DTFINVALOBJET' => $params['P_DTFINVALOBJET'],
    		    'P_NRORG'       => $this->environment->getNrorg()
            );
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir o preço objetivo!');
        }
    }
    
    public function insertProdPlanObj($params){    
        try {
            $query = "INSERT INTO ITEMPRECOJETIVO (CDFILIAL, NRPRECOBJET, CDREGIAO, CDPRODUTO, DTINIVALITMOBJ, DTFINVALITMOBJ, VRPRECOBJETIVO, NRORG) VALUES (:CDFILIAL, :NRPRECOBJET, :CDREGIAO, :CDPRODUTO, :DTINIVALITMOBJ, :DTFINVALITMOBJ, :VRPRECOBJETIVO, :P_NRORG)";
           
            $param = array (
    		    'CDFILIAL'       => $params['P_CDFILIAL'],
    		    'CDREGIAO'       => $params['P_CDREGIAO'],
    		    'NRPRECOBJET'    => $params['P_NRPRECOBJET'],
    		    'CDPRODUTO'      => $params['P_CDPRODUTO'],
    		    'DTINIVALITMOBJ' => $params['P_DTINIVALOBJET'],
    		    'DTFINVALITMOBJ' => $params['P_DTFINVALOBJET'],
    		    'VRPRECOBJETIVO' => $params['P_VRPRECOBJETIVO'],
    		    'P_NRORG'        => $this->environment->getNrorg()
            );
            
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir o item do preço objetivo!');
        }
    }
    
    
    /********************
     *      DELETES     *
     * ******************/
     
    public function deletaplanilha($params){   
        try {
            $query = "DELETE FROM PRECOBJETIVO WHERE CDFILIAL = :OLD_CDFILIAL AND NRPRECOBJET = :OLD_NRPRECOBJET AND DTINIVALOBJET = :OLD_DTINIVALOBJET AND DTFINVALOBJET = :OLD_DTFINVALOBJET AND NRORG = :OLD_NRORG";
            
            $params = [
                'OLD_CDFILIAL'      => $params['P_CDFILIAL'],
                'OLD_NRPRECOBJET'   => $params['P_NRPRECOBJET'],
                'OLD_DTINIVALOBJET' => $params['P_DTINIVALOBJET'],
                'OLD_DTFINVALOBJET' => $params['P_DTFINVALOBJET'],
                'OLD_NRORG'         => $this->environment->getNrorg()
            ];

            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir o preço objetivo!');
        }
    }
    
    public function deletaProduto($params){   
        try {
            $query = "DELETE FROM ITEMPRECOJETIVO WHERE CDFILIAL = :OLD_CDFILIAL AND NRPRECOBJET = :OLD_NRPRECOBJET AND CDREGIAO = :OLD_CDREGIAO AND CDPRODUTO = :OLD_CDPRODUTO AND NRORG = :OLD_NRORG";
            
            $params = [
                'OLD_CDFILIAL'    => $params['P_CDFILIAL'],
                'OLD_NRPRECOBJET' => $params['P_NRPRECOBJET'],
                'OLD_CDREGIAO'    => $params['P_CDREGIAO'],
                'OLD_CDPRODUTO'   => $params['P_CDPRODUTO'],
                'OLD_NRORG'       => $this->environment->getNrorg()
            ];

            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao excluir o produto!');
        }
    }
}